//********************************************************************
//*** ResourcePool/SQL/ResourcePool.hh
//*** Copyright (c) 2003-2009 by Markus Winand <mws@fatalmind.com>
//*** $Id: ResourcePool.hh,v 1.17 2009-03-02 14:16:45 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#ifndef RESOURCEPOOL_SQL_RESOURCEPOOL_HH
#define RESOURCEPOOL_SQL_RESOURCEPOOL_HH

#include "ResourcePool/ResourcePoolType.hh"
#include "ResourcePool/IgnoreExceptionCommand.hh"
#include "ResourcePool/SQL/Factory.hh"
#include "ResourcePool/SQL/Resource.hh"

#include "ResourcePool/SQL/RTSQLExecute.hh"
#include "ResourcePool/SQL/RTSQLSelect.hh"
#include "ResourcePool/SQL/RTSQLTransaction.hh"
#include "ResourcePool/SQL/RTSQLBatch.hh"

namespace fatalmind {
/*
 * the fatalmind::SQL namespace serves two requirements:
 * - it includes the runtime selectable SQL resource type
 * - it includes code common to all SQL resource types
 */
namespace SQL {

template<class TM>
struct Commands;

/**
 * The pool type to access SQL databases with runtime type.
 */
template<class TM>
class ResourceType
: public fatalmind::PoolType<SQLFactory<TM> 
	, DummySQLResource<TM>
	, SQLResource<TM> // plain resource
	, Commands<TM>
	, TM>
{
};

template<class TM>
struct Commands {
	Commands(const SQLFactory<TM>& f) 
	: type_(f.getDatabaseType())
	{
	};

	CommandParameters<TM> SQL(const std::string& sql) {
		CommandParameters<TM> cp(type_, sql);
		return cp;
	}
	
	typedef Command<ResourceType<TM> >   Base;
	typedef RTSQLExecute<TM>   SQLExecute;
	typedef RTSQLSelect<TM>    SQLSelect;
	typedef RTSQLSelectRow<TM> SQLSelectRow;
	typedef RTSQLBatch<TM>     SQLBatch;
	typedef RTSQLTransaction<TM>  Transaction;
	typedef fatalmind::IgnoreExceptionCommand<RTSQLExecute<TM> >  IgnoreException;
	private:
		const typename SQLFactory<TM>::databaseType_t type_;
};

} // namespace SQL

namespace ResourceType {

typedef SQL::ResourceType<DefaultThreadedModel > SQL;
typedef SQL::ResourceType<SingleThreadedModel<> > SQLST;
typedef SQL::ResourceType<MultiThreadedModel<> > SQLMT;

} // namespace ResourceType
} // namespace fatalmind

#endif
