//********************************************************************
//*** ResourcePool/mysql/SQLType.cc
//*** Copyright (c) 2003-2009 by Markus Winand <mws@fatalmind.com>
//*** $Id: SQLType.cc,v 1.13 2009-03-02 14:16:45 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#ifndef RESOURCEPOOL_MYSQL_SQLTYPE_CC
#define RESOURCEPOOL_MYSQL_SQLTYPE_CC

#ifndef RESOURCEPOOL_MYSQL_SQLTYPE_HH
#include "SQLType.hh"
#endif

#include "ResourcePool/SQL/BindOutInterface.hh"

#ifndef RESOURCEPOOL_MYSQL_MYSQLEXCEPTION_HH
#include "MysqlException.hh"
#endif

#include <sstream>
#include <limits>

namespace fatalmind {
namespace mysql {

template<typename T>
SQLTypeImpl<T>::SQLTypeImpl(const T& val) 
: _val(val)
{
}

template<typename T>
SQLTypeImpl<T>::~SQLTypeImpl() 
{
}

template<typename T>
bool SQLTypeImpl<T>::needsquote() const 
{
	return false;
}

template<typename T>
void SQLTypeImpl<T>::outputoperator(std::ostream& os) const
{
	os << _val;
}

/*
 * String specialization
 */

template<>
bool SQLTypeImpl<std::string>::needsquote() const
{
	return true;
}

/*
 * Instantiation
 */

template class SQLTypeImpl<char>;
template class SQLTypeImpl<signed char>;
template class SQLTypeImpl<unsigned char>;

template class SQLTypeImpl<short int>;
template class SQLTypeImpl<int>;
template class SQLTypeImpl<long int>;

template class SQLTypeImpl<unsigned short int>;
template class SQLTypeImpl<unsigned int>;
template class SQLTypeImpl<unsigned long int>;

// wchar_t, bool not supported

template class SQLTypeImpl<float>;
template class SQLTypeImpl<double>;
template class SQLTypeImpl<long double>;
template class SQLTypeImpl<std::string>;

/*
 *
 * SQLRefType
 *
 */

template<typename T>
SQLRefTypeImpl<T>::SQLRefTypeImpl(T& val, bool& a_isnull)
: val(val)
, isnull(a_isnull)
{
}

template<typename T>
SQLRefTypeImpl<T>::~SQLRefTypeImpl()
{
}

template<typename T>
void SQLRefTypeImpl<T>::assign(const char* chr, const int len) 
{
	if (chr == 0) {
		bool& notnull = SQL::BindOutInterface::notnullconstant;
		if (&isnull == &notnull) {
			// TODO: the SQL statement is actually known, somewhere else
			throw NFOMysqlException("Indicator variable required but not supplied.", FATALMIND_MYSQL_ERRCODE_BINDOUTNOTNULL, "UKN");
		} 
		isnull = true;
	} else {
		isnull = false;

		_valptr = chr;
		std::string str(chr, len);
		std::istringstream is(str);
		if (!(is >> val)) {
			// TODO: the SQL statement is actually known, somewhere else
			throw NFOMysqlException("value does not fit into specified type", 98, "UKN");
		}
		/* really strange, the above code, fails to fail for unsigned int and unsigned long
		 * but works for unsiged short
		 * maybe a compiler/library problem (problem happend with GCC 3.3.1)
		 * however, for now i will do the check manually */
		if (str[0] == '-' && ! std::numeric_limits<T>::is_signed) {
			// TODO: the SQL statement is actually known, somewhere else
			throw NFOMysqlException("value does not fit into specified type", 98, "UKN");
		}
	}
}

template<>
void SQLRefTypeImpl<std::string>::assign(const char* chr, const int len)
{
	if (chr == 0) {
		bool& notnull = SQL::BindOutInterface::notnullconstant;
		if (&isnull == &notnull) {
			// TODO: the SQL statement is actually known, somewhere else
			throw NFOMysqlException("Indicator variable required but not supplied.", FATALMIND_MYSQL_ERRCODE_BINDOUTNOTNULL, "UKN");
		} 
		isnull = true;
	} else {
		isnull = false;
		_valptr = chr;
		val.assign(chr, len);
	}
}

/*
template<typename T>
std::istream& SQLRefTypeImpl<T>::opistream(std::istream& is)
{
	return is >> val;
}
*/

/*
 * Instantioation
 */

template class SQLRefTypeImpl<char>;
template class SQLRefTypeImpl<signed char>;
template class SQLRefTypeImpl<unsigned char>;

template class SQLRefTypeImpl<short int>;
template class SQLRefTypeImpl<int>;
template class SQLRefTypeImpl<long int>;

template class SQLRefTypeImpl<unsigned short int>;
template class SQLRefTypeImpl<unsigned int>;
template class SQLRefTypeImpl<unsigned long>;

// wchar_t and bool not supported

template class SQLRefTypeImpl<float>;
template class SQLRefTypeImpl<double>;
template class SQLRefTypeImpl<long double>;

template class SQLRefTypeImpl<std::string>;

} // namespace mysql
} // namespace fatalmind
#endif
