//********************************************************************
//*** ResourcePool/oracle/SQLSelect.cc
//*** Copyright (c) 2003-2009 by Markus Winand <mws@fatalmind.com>
//*** $Id: SQLSelect.cc,v 1.42 2009-03-02 14:16:46 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of 
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#ifndef RESOURCEPOOL_ORACLE_SQLSELECT_HH
#include "SQLSelect.hh"
#endif

#ifndef RESOURCEPOOL_ORACLE_RESOURCE_HH
#include "Resource.hh"
#endif

#ifndef RESOURCEPOOL_ORACLE_ORACLETYPE_HH
#include "oracleType.hh"
#endif

#ifndef RESOURCEPOOL_ORACLE_ORACLEREFTYPE_HH
#include "oracleRefType.hh"
#endif

#ifndef RESOURCEPOOL_ORACLE_ORACLEEXCEPTIONFACTORY_HH
#include "OracleExceptionFactory.hh"
#endif

#ifndef RESOURCEPOOL_ORACLE_SELECTLISTMETADATA_HH
#include "SelectListMetaData.hh"
#endif
//#include "OCIHandle.hh"

#ifndef RESOURCEPOOL_ORACLE_COMMANDINTERFACE_CC
#include "CommandInterface.cc"
#endif

#ifndef RESOURCEPOOL_ORACLE_RESOURCEPOOL_HH
#include "ResourcePool.hh"
#endif

namespace fatalmind {
namespace oracle {

template<class TM, template<class> class SLMDC>
const std::string SQLSelect<TM, SLMDC>::_SQLSelectName("oracle::SQLSelect");

template<class TM, template<class> class SLMDC>
SQLSelect<TM, SLMDC>::SQLSelect(const std::string& SQL, SQL::SQLFetcher& fetcher) 
: SQLSelectRow<TM, SLMDC>(SQL, fetcher)
{
}

template<class TM, template<class> class SLMDC>
SQLSelect<TM, SLMDC>::SQLSelect(const ResourcePool<ResourceType<TM> >& p, const std::string& SQL, SQL::SQLFetcher& fetcher) 
: SQLSelectRow<TM, SLMDC>(SQL, fetcher)
{
}

template<class TM, template<class> class SLMDC>
SQLSelect<TM, SLMDC>::SQLSelect(const SQLSelect& rho) 
: SQLSelectRow<TM, SLMDC>(rho) 
{
}

template<class TM, template<class> class SLMDC>
SQLSelect<TM, SLMDC>::~SQLSelect()
{
}

template<class TM, template<class> class SLMDC>
void SQLSelect<TM, SLMDC>::_execute(oracleResource<TM>& rhand, StatementCache& sthcache) 
{
	oracleHandlesWithCache<TM>& hand = rhand.get_plain_resource();
	sword ret;
	bool slmdchit = prepare_buffer(rhand, sthcache);

	real_bind(hand);
	preSelectBufferBind(hand);

	/* execute statement */
	if ((ret = OCIStmtExecute(hand.getSvcCtx()
		, _sth->get()
		, hand.getError()
		, (ub4) 0
		, (ub4) 0
		, (CONST OCISnapshot *) 0
		, (OCISnapshot *) 0
		, (ub4) ((hand.getTxDepth() == 0) ? OCI_COMMIT_ON_SUCCESS : OCI_DEFAULT))
	) != OCI_SUCCESS) {
		OracleExceptionFactory oef("OCIStmtExecute", hand.getError(), ret, _SQL);
		 rhand.event(rhand.Event.EXECUTE, oef.getErrorCode());
		_sth.reset();
		oef.throwit();
	}
	rhand.event(rhand.Event.EXECUTE, slmdchit ? "SLMDC" : "");

	internal::registerFinish fin(hand, *this, *_sth, _SQL);
	postSelectBuffer(rhand);

	bool more = true;
	int  row  = 0;
	OCIStatementHandle& p_sth = *_sth; // do not always dereference
	while ((more = fetch(hand, p_sth, _SQL)) && ++row && fetcher.process(row)) {
		 rhand.event(rhand.Event.FETCH, "");
	}
	fetcher.done(more, row);
	_sth.reset();
}

template<class TM, template<class> class SLMDC>
const std::string& SQLSelect<TM, SLMDC>::getCommandName() const {
	return _SQLSelectName;
}

template class SQLSelect<DefaultThreadedModel, oracle::SelectListMetaDataCache >;
//template class SQLSelect<DefaultThreadedModel, oracle::SelectListMetaDataNopCache >;

} // namespace oracle
} // namespace fatalmind
